/*-----------------------------------------------------------------------*/
/* filename -       TDemo.cpp                                            */
/*                                                                       */
/* function(s)                                                           */
/*                  definitions for TDemo class                          */
/*                  demonstrates use of the TVColorDialog library        */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/*                                                                       */
/*    TDEMO.CPP                                                          */
/*                                                                       */
/*    Copyright (C) 1992 Michael Newton and Comsoft Software             */
/*    All Rights Reserved.                                               */
/*                                                                       */
/*-----------------------------------------------------------------------*/

#define Uses_TApplication
#define Uses_TBackground
#define Uses_TButton
#define Uses_TCheckBoxes
#define Uses_TCollection
#define Uses_TColorGroup
#define Uses_TColorItem
#define Uses_TDeskTop
#define Uses_TDialog
#define Uses_TEvent
#define Uses_TKeys
#define Uses_TLabel
#define Uses_TMenu
#define Uses_TMenuBar
#define Uses_TMenuItem
#define Uses_TObject
#define Uses_TPalette
#define Uses_TRect
#define Uses_TScreen
#define Uses_TStatusDef
#define Uses_TStatusItem
#define Uses_TStatusLine
#define Uses_TStaticText
#define Uses_TSItem
#define Uses_TSubMenu
#define Uses_TStreamable
#define Uses_TStreamableClass
#define Uses_TView
#define Uses_TWindow
#define Uses_ipstream
#define Uses_opstream
#define Uses_fpstream
#define Uses_MsgBox

#include <tv.h>

__link(RMenuBar)
__link(RPatternButtons)
__link(RPatternCluster)
__link(RRadioButtons)
__link(RResourceCollection)
__link(RStaticText)
__link(RVColorDialog)


// CRTL header files ******************************************************

#if !defined __STDLIB_H
#include <stdlib.h>
#endif

#if !defined __STDIO_H
#include <stdio.h>
#endif

#if !defined __STDARG_H
#include <stdarg.h>
#endif

#if !defined __CONIO_H
#include <conio.h>
#endif

#if !defined __IOSTREAM_H
#include <iostream.h>
#endif

#if !defined __STRSTREA_H
#include <strstrea.h>
#endif

#if !defined __FSTREAM_H
#include <fstream.h>
#endif

#if !defined __IOMANIP_H
#include <iomanip.h>
#endif

#if !defined __STRING_H
#include <string.h>
#endif

#if !defined __DOS_H
#include <dos.h>
#endif

#if !defined __DIR_H
#include <dir.h>
#endif

#if !defined __ALLOC_H
#include <alloc.h>
#endif


// non-CRTL header files **************************************************

#if !defined __TDEMO_HPP
#include "tdemo.hpp"
#endif

#if !defined __COLORDLG_HPP
#include "colordlg.hpp"
#endif


// External variables *****************************************************

extern TPoint shadowSize;


// TMenuItem **************************************************************

TMenuItem& operator +(TMenuItem& item1, TMenuItem& item2)
{
   TMenuItem *temp = &item1;
   while(temp->next != NULL)
      temp = temp->next;
   temp->next = &item2;
   return item1;
}


// TDemo ******************************************************************

const char * near TDemo::configFile = "TDEMO.CFG";

TDemo::TDemo() : TProgInit(initStatusLine, initMenuBar, initDeskTop)
{
   // This is the default palette for the application. If no configuration
   // file is found, this palette will be copied into the applications
   // palette.
   // This palette was created by selecting "savePalette" on the menubar.
   // SavePalette creates a text file named PALETTE.TXT, which will
   // consist of an array similar to "defpal" below. The attributes in
   // PALETTE.TXT will be the currently selected application's colors.

   uchar defPal[] = {0x09,0x17,0x18,0x1F,0x0E,0x04,0x0F,0x10,0x17,0x0E,
                     0x09,0x1A,0x17,0x1C,0x1E,0x37,0x3F,0x0E,0x0C,0x13,
                     0x30,0x3C,0x00,0x78,0x70,0x0E,0x0E,0x13,0x70,0x40,
                     0x1E,0x78,0x70,0x0E,0x6E,0x4E,0x70,0x70,0x7E,0x7E,
                     0x30,0x3E,0x3F,0x37,0x3B,0x78,0x70,0x7E,0x7E,0x03,
                     0x4F,0x7E,0x5E,0x55,0x0B,0x3E,0x03,0x4F,0x0E,0x07,
                     0x0F,0x00,0x00,0x37,0x3F,0x3A,0x13,0x13,0x30,0x3E,
                     0x1E};

   // Call the restoreDesk function to restore the desktop
   // If a valid configuration file exists, restoreDesk will restore
   // the desktop. If not, then we set the defaults
   if(!restoreDesk())
      {
      // Set background pattern character to ascii 206 
      deskTop->background->setPattern(206);
      // Save the pattern character in member variable
      currentPattern = deskTop->background->getPattern();
      // TPatternButtons start at 4
      patternButtons = 4;
      // Copy the default palette into the application's palette
      TPalette &palette = getPalette();
      ushort length = palette[0];
      memmove(&palette[1], &defPal[0], (size_t) length);
      // Redraw the desktop
      setState(sfExposed, False);
      setState(sfExposed, True);
      redraw();
      }
}


TDemo::~TDemo()
// For debugging only
{
}


TMenuBar *TDemo::initMenuBar(TRect r)
{
   TMenuItem *sys =
   new TMenuItem("~\360~", cmAbout, kbAltSpace, hcNone);
   TMenuItem *exit =
   new TMenuItem("E~x~it", cmQuit, kbAltX, hcNone);
   TMenuItem *colors =
   new TMenuItem("~C~olors", cmColors, kbNone, hcNone);
   TMenuItem *pal =
   new TMenuItem("~W~rite palette", cmSavPal, kbAltP, hcNone);

   TMenu *menu = new TMenu(*sys + *exit + *colors + *pal);
   r.b.y = r.a.y + 1;

   return(new TMenuBar(r, menu));
}


TStatusLine *TDemo::initStatusLine(TRect r)
{
   r.a.y = r.b.y - 1;

   return new TStatusLine(r,
      *new TStatusDef(0, 0xffff) +
      *new TStatusItem("~Alt-X~ Exit", kbAltX, cmQuit) +
      *new TStatusItem("~F10~ Menu", kbF10, cmMenu));
}


void TDemo::handleEvent(TEvent& event)
{
   // If user selects "Exit", save the current desktop
   if(event.what == evCommand && event.message.command == cmQuit)
      {
      if(!confirmExit())          // Get confirmation
         clearEvent(event);
      else
         saveDesk();              // Save desktop before exiting
      }

   TApplication::handleEvent(event);

   switch(event.what)
      {
      case evCommand:
         switch(event.message.command)
            {
            case cmSavPal:
               writePalette("palette.txt");
               break;
            case cmAbout:
               about();
               break;
            case cmColors:
               colors();
               break;
            }
         break;
      default:
         return;
      }

   clearEvent(event);
}


void TDemo::about()
{
   char *line[] = {
                  "TVColorDialog Demo",
                  "(C) Copyright 1992",
                  "Michael Newton and Comsoft Software",
                  "All rights reserved.",
                  "Demonstrates the TVColorDialog class."
                  };

   TDialog *d = new TDialog(TRect(0, 0, 56, 13), "About");

   TStaticText *t;
   int len = strlen(line[0]);
   t = new TStaticText(TRect(0, 2, len, 3), line[0]);
   t->options |= ofCenterX;
   d->insert(t);

   len = strlen(line[1]);
   t = new TStaticText(TRect(0, 4, len, 5), line[1]);
   t->options |= ofCenterX;
   d->insert(t);

   len = strlen(line[2]);
   t = new TStaticText(TRect(0, 5, len, 6), line[2]);
   t->options |= ofCenterX;
   d->insert(t);

   len = strlen(line[3]);
   t = new TStaticText(TRect(0, 6, len, 7), line[3]);
   t->options |= ofCenterX;
   d->insert(t);

   len = strlen(line[4]);
   t = new TStaticText(TRect(0, 8, len, 9), line[4]);
   t->options |= ofCenterX;
   d->insert(t);

   d->insert(new TButton(TRect(4, 10, 52, 12),
             "~O~K", cmCancel, bfDefault));

   d->options |= ofCentered;

   if(validView(d))
      deskTop->execView(d);

   destroy(d);
}


void TDemo::colors()
{
   // The pattern characters
   const uchar patternChar[MAXPATTERNS] =
                  {176, 177, 178, 32, 206, 190, 197,
                   219, 216, 215, 221, 208, 223};
   // The pattern character strings
   char patternString[MAXPATTERNS][2];

   // Put the pattern characters in the pattern strings
   for(int ctr = 0; ctr < MAXPATTERNS; ctr++)
      {
      patternString[ctr][0] = patternChar[ctr];
      patternString[ctr][1] = '\0';
      }

   // Build the list of TSItems for the pattern selector radio buttons
   // using the pattern strings
   TSItem *items = 0;
   for(ctr = MAXPATTERNS - 1; ctr >= 0; ctr--)
      items = new TSItem(patternString[ctr], items);

   // Create color groups and items
   TColorGroup &group1 =
      *new TColorGroup("Desktop")                 +
         *new TColorItem("Color",              1);

   TColorGroup &group2 =
      *new TColorGroup("Menus")                   +
         *new TColorItem("Normal",             2) +
         *new TColorItem("Disabled",           3) +
         *new TColorItem("Shortcut",           4) +
         *new TColorItem("Selected",           5) +
         *new TColorItem("Selected disabled",  6) +
         *new TColorItem("Shortcut selected",  7);

   TColorGroup &group3 =
      *new TColorGroup("Blue Windows")            +
         *new TColorItem("Frame/passive",      8) +
         *new TColorItem("Frame/active",       9) +
         *new TColorItem("Frame icons",       10) +
         *new TColorItem("Scrollbar",         11) +
         *new TColorItem("Normal text",       13) +
         *new TColorItem("Selected text",     14);

   TColorGroup &group4 =
      *new TColorGroup("Cyan Windows")            +
         *new TColorItem("Frame/passive",     16) +
         *new TColorItem("Frame/active",      17) +
         *new TColorItem("Frame icons",       18) +
         *new TColorItem("Scrollbar",         19) +
         *new TColorItem("Normal text",       21) +
         *new TColorItem("Selected text",     22);

   TColorGroup &group5 =
      *new TColorGroup("Gray Windows")            +
         *new TColorItem("Frame/passive",     24) +
         *new TColorItem("Frame/active",      25) +
         *new TColorItem("Frame icons",       26) +
         *new TColorItem("Scrollbar",         27) +
         *new TColorItem("Normal text",       29) +
         *new TColorItem("Selected text",     30);

   TColorGroup &group6 =
      *new TColorGroup("Dialogs")                 +
         *new TColorItem("Frame/passive",     32) +
         *new TColorItem("Frame/active",      33) +
         *new TColorItem("Frame icons",       34) +
         *new TColorItem("Scrollbar",         35) +
         *new TColorItem("Scrollbar icons",   36) +
         *new TColorItem("Static text",       37) +
         *new TColorItem("Label normal",      38) +
         *new TColorItem("Label highlight",   39) +
         *new TColorItem("Label shortcut",    40);

         TColorItem &dialogItems1 =
         *new TColorItem("Button normal",     41) +
         *new TColorItem("Button default",    42) +
         *new TColorItem("Button selected",   43) +
         *new TColorItem("Button disabled",   44) +
         *new TColorItem("Button shortcut",   45) +
         *new TColorItem("Button shadow",     46) +
         *new TColorItem("Cluster normal",    47) +
         *new TColorItem("Cluster selected",  48) +
         *new TColorItem("Cluster shortcut",  49);

         TColorItem &dialogItems2 =
         *new TColorItem("Input normal",      50) +
         *new TColorItem("Input selected",    51) +
         *new TColorItem("Input arrows",      52) +
         *new TColorItem("History arrow",     53) +
         *new TColorItem("History sides",     54) +
         *new TColorItem("History scrollbar", 55) +
         *new TColorItem("History icons",     56) +
         *new TColorItem("List normal",       57) +
         *new TColorItem("List focused",      58) +
         *new TColorItem("List selected",     59) +
         *new TColorItem("List divider",      60) +
         *new TColorItem("Information pane",  61);

   group6 = group6 + dialogItems1 + dialogItems2;

   TColorGroup &groups = group1 + group2 + group3 +
                         group4 + group5 + group6;

   // Create the TVColorDialog. addHelpButton = True
   TVColorDialog *d;
   d = new TVColorDialog((TPalette *) 0, &groups, items, True);

   // Set the dialog's help context to none
   // (For demonstration use only)
   d->helpCtx = hcColors;

   if(validView(d) != 0)                         // If it's valid...
      {
      d->setData(&getPalette());
      d->patSel->setData(&patternButtons);
      if(deskTop->execView(d) != cmCancel)       // exec it
         {
         // getPalette now points to new palette
         getPalette() = *(d->pal);
         // Get pattern character from dialog's patSel member
         d->patSel->getData(&patternButtons);
         // Set background pattern character
         deskTop->background->setPattern(patternChar[patternButtons]);
         // Store the selected pattern character
         currentPattern = patternChar[patternButtons];
         // Redraw
         setState(sfExposed, False);
         setState(sfExposed, True);
         redraw();
         }
      destroy(d);
      }
}


Boolean TDemo::saveDesk()
// Saves the desktop to a file by calling the storeDesktop function
{
   char msg[81];
   ostrstream os(msg, sizeof msg);

   fpstream *f = new fpstream(configFile, ios::trunc | ios::binary);

   if(!f)
      {
      os << "\003Could not create \""
         << configFile
         << "\""
         << ends;
      messageBox(os.str(), mfOKButton | mfError);
      return False;
      }

   opstream& strm = *f;

   // Store the desktop pattern character
   strm.writeBytes(&currentPattern, sizeof currentPattern);

   // Store patternButtons value
   strm.writeBytes(&patternButtons, sizeof patternButtons);

   // Store the current palette
   TPalette &palette = getPalette();
   ushort length = palette[0];
   strm.writeBytes(&palette[0], sizeof(palette[0]));
   for(int ctr = 1; ctr < length + 1; ctr++)
      strm.writeBytes(&palette[ctr], sizeof(palette[ctr]));

   // Store current video mode
   ushort mode = TScreen::screenMode;
   strm.writeBytes(&mode, sizeof mode);

   // Call storeDesk to stream all views
   TDemo::storeDesk(*f);
   if(!f)
      {
      os << "\003Could not create \""
         << configFile
         << "\""
         << ends;
      messageBox(os.str(), mfOKButton | mfError);
      delete f;
      ::remove(configFile);
      return False;
      }

   delete f;
   return True;
}


Boolean TDemo::restoreDesk()
// Restores the desktop config file
{
   ffblk ffblk;
   char msg[81];
   ostrstream os(msg, sizeof msg);

   if(findfirst(configFile, &ffblk, 0))
      return False;

   fpstream *f = new fpstream(configFile, ios::in | ios::binary);

   if(!f)
      {
      os << "\003Could not open configuation file \""
         << configFile
         << "\""
         << ends;
      messageBox(os.str(), mfOKButton | mfError);
      return False;
      }

   ipstream& strm = *f;

   // Get the desktop pattern character
   strm.readBytes(&currentPattern, sizeof currentPattern);

   // Get patternButtons value
   strm.readBytes(&patternButtons, sizeof patternButtons);

   // Set background pattern character and redraw
   deskTop->background->setPattern(currentPattern);
   setState(sfExposed, False);
   setState(sfExposed, True);
   redraw();

   // Get the palette
   TPalette &palette = getPalette();
   ushort length = palette[0];
   strm.readBytes(&palette[0], sizeof(palette[0]));
   for(int ctr = 1; ctr < length + 1; ctr++)
      strm.readBytes(&palette[ctr], sizeof(palette[ctr]));

   // Get the video mode
   ushort mode;
   strm.readBytes(&mode, sizeof mode);

   // Set shadow size
   if((mode & TDisplay::smFont8x8) != 0)
      shadowSize.x = 1;
   else
      shadowSize.x = 2;

   // Set video mode
   setScreenMode(mode);

   // Load the desktop views
   TDemo::loadDesk(*f);
   if(!f)
      {
      os << "\003Error reading configuration file \""
         << configFile
         << "\""
         << ends;
      messageBox(os.str(), mfOKButton | mfError);
      return False;
      }

   delete f;
   return True;
}


void TDemo::storeDesk(fpstream& strm)
// Stores the Desktop views in a resource file
{
   deskTop->forEach(::writeView, &strm);
   strm << 0;
}


void TDemo::loadDesk(fpstream &strm)
// Extracts all views from fpstream strm and inserts them into desktop
{
   TView *v;
   if(deskTop->valid(cmClose))
      {
      deskTop->forEach(::closeView, 0);     // Clear the desktop
      setState(sfExposed, False);           // Free view buffers
      setState(sfExposed, True);            // Back to normal
      redraw();                             // Redraw it all
      do
         {
         strm >> v;
         deskTop->insertBefore(validView(v), deskTop->last);
         }
      while(v != 0);
      }
}


void TDemo::writePalette(const char *path)
// Writes the applications palette (formatted) to a text file
{
   char msg[81];
   ostrstream os(msg, sizeof msg);

   fpstream *f = new fpstream(path, ios::out | ios::trunc);
   if(!f)
      {
      os << "\003Could not create \""
         << path
         << "\""
         << ends;
      messageBox(os.str(), mfOKButton | mfError);
      exit(1);
      }
   else
      {
      opstream& strm = *f;
      TPalette &palette = getPalette();
      ushort len = palette[0];
      char oldFill;
      os.seekp(0);
      os << "   uchar defPal[] = {"
         << ends;
      strm.writeBytes(os.str(), strlen(msg));
      os.setf(ios::uppercase);
      os.setf(ios::hex, ios::basefield);
      oldFill = os.fill('0');
      for(int ctr = 1; ctr < len + 1; ctr++)
         {
         os.seekp(0);
         os << "0x";
         os.width(2);
         os << (unsigned int) palette[ctr];
         if(ctr == len)
            os << ends;
         else if(ctr % 10 == 0)
            os << ",\n                     " << ends;
         else
            os << "," << ends;
         strm.writeBytes(os.str(), strlen(msg));
         }
      os.fill(oldFill);
      os.seekp(0);
      os << "}\;\n" << ends;
      strm.writeBytes(os.str(), strlen(msg));
      f->close();
      delete f;
      }
}



// Non-member functions ***************************************************

static void writeView(TView *v, void *strm)
// Writes a view object to a resource file
{
   fpstream *s = (fpstream *) strm;
   if(v != TProgram::deskTop->last)
      *s << v;
}


static void closeView(TView *v, void *p)
// Broadcast cmClose message to v
{
   message(v, evCommand, cmClose, p);
}


ushort execDialog(TDialog *d, void *data)
// Execute TDialog d with setData and getData
{
   TView *p = TProgram::application->validView(d);
   if(p == 0)
      return cmCancel;
   else
      {
      if(data != 0)
         p->setData(data);
      ushort result = TProgram::deskTop->execView(p);
      if(result != cmCancel && data != 0)
         p->getData(data);
      TObject::destroy(p);
      return result;
      }
}


Boolean confirmExit()
{
   TDialog *d = new TDialog(TRect(0, 0, 38, 7), "Exit");

   char msg[] = "Are you sure you wish to exit?";
   TStaticText *t = new TStaticText(TRect(0, 2, 30, 3), msg);
   t->options |= ofCenterX | ofFramed;
   d->insert(t);

   TButton *b;
   b = new TButton(TRect(4, 4, 17, 6), "~Y~es", cmOK, bfDefault);
   d->insert(b);
   b = new TButton(TRect(20, 4, 34, 6), "~N~o", cmCancel, bfNormal);
   d->insert(b);

   d->flags ^= wfMove;
   d->options |= ofCentered;
   d->selectNext(True);

   ushort result = execDialog(d, 0);

   if(result == cmCancel)
      return False;
   else
      return True;
}


int main()
{
   TDemo program;
   program.run();
   return(0);
}


//                                End of TDEMO.CPP

